=begin
Copyright(C) 2010, kyyu
All Rights Reserved
Permission to use, copy, modify, and distribute this software for any purpose
and without fee is hereby granted, provided this notice appears in any copies.
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED 
WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

Name:      		Slots 4 Tabs
Version:   		1.1
SU Version:   	7.0
Date:      		06-02-2010

Description:   	Automatically makes mating slots for a tabbed part
Usage:      	Select both the parts (each grouped) and right click context menu ( [kyyu]/Slots 4 Tabs )
                * Plugin will automatically detect which part is which
                * Parts should be place in proper perpendicular, intersecting position
                * Parts are assumed to be flat plates, thickness same as tab height
                * Tab height needs to match expected thickness exactly (default is 0.25 inch)
                * How to change thickness value:
                    1)  Type for example "$kyyu_s4t = 0.25 or 3.mm" into the Ruby Console.  Will be good for entire SU session.
                    2)  Or change the default permanently, a few lines down in the code.
                * Plugin automatically defaults to the move tool when done.  You can comment this out, if not wanted.

History:
			1.0 (05-27-2010)  -  first version
            1.1 (06-02-2010)  -  fixed some metric bugs
	
=end  

require 'sketchup.rb'
$kyyu_s4t = 0.25

def kyyu_Slots_4_Tabs


# if $kyyu_s4t_set != 1 then $kyyu_s4t = 0.25 end
# $kyyu_s4t_set = 1
puts "Expected thickness = #{$kyyu_s4t}"
model = Sketchup.active_model
ent = model.entities
sel = model.selection
view = Sketchup.active_model.active_view

# Selection error check
sel_count = sel.length
if sel_count < 2 
    UI::messagebox ("ERROR - Less than 2 entities selected")
    return  
end
selg = [] 
for x in 0...sel_count
    if sel[x].is_a? Sketchup::Group
        selg.push sel[x]
    end
end
selg_count = selg.length 
if selg_count < 2 
    UI::messagebox ("ERROR - You need to select 2 groups")
    return   
end 
if selg_count > 2 
    UI::messagebox ("ERROR - You have selected more than 2 groups")
    return    
end 
puts "I have cleared sel error section."

# Tr detection
temp_b1 = selg[0].bounds
temp_b2 = selg[1].bounds
temp_bb = temp_b1.intersect temp_b2
temp_v = temp_bb.height * temp_bb.width * temp_bb.depth
puts "nonTr bb_int vol = #{temp_v}"

Sketchup.active_model.start_operation "ky_slots4tabs" 
vol = []
for x in 0...2
    bbtr = selg[x].transformation
    Sketchup.active_model.active_entities.transform_entities(bbtr.inverse,selg[0],selg[1])
    temp_b1 = selg[0].bounds
    temp_b2 = selg[1].bounds
    temp_bb = temp_b1.intersect temp_b2
    vol[x] = temp_bb.height * temp_bb.width * temp_bb.depth
    puts "Tr#{x+1} bb_int vol = #{vol[x]}"
    #UI::messagebox ("bb check")
    Sketchup.active_model.active_entities.transform_entities(bbtr,selg[0],selg[1])    
end
if vol[0] <= vol[1]
    bbtr = selg[0].transformation
    movegroup = selg[0]
else
    bbtr = selg[1].transformation
    movegroup = selg[1]
end

    Sketchup.active_model.active_entities.transform_entities(bbtr.inverse,selg[0],selg[1])
    temp_b1 = selg[0].bounds
    temp_b2 = selg[1].bounds
    temp_bb = temp_b1.intersect temp_b2
    vol_chosen = temp_bb.height * temp_bb.width * temp_bb.depth
    puts "Tr **Chosen** bb_int vol = #{vol_chosen}"
    # UI::messagebox ("bb check")
    # Sketchup.active_model.active_entities.transform_entities(bbtr,selg[0],selg[1])    



# Part detection
g1 = selg[0]
g2 = selg[1]
g1_ent = g1.entities
g2_ent = g2.entities
b1 = g1.bounds
b2 = g2.bounds
bb = b1.intersect b2

qnum1 = 0
qnum2 = 0
g1_ent.each do |e|
	if e.is_a? Sketchup::Edge
	p1 = e.start.position.transform!(g1.transformation)
	#p2 = e.end.position.transform!(g1.transformation)
		if ((e.length == $kyyu_s4t) && (bb.contains? p1))
			qnum1 = qnum1 + 1
		end
	end
end	
g2_ent.each do |e|
  if e.is_a? Sketchup::Edge
  p1 = e.start.position.transform!(g2.transformation)
  #p2 = e.end.position.transform!(g2.transformation)
    if ((e.length == $kyyu_s4t) && (bb.contains? p1))
    qnum2 = qnum2 + 1
    end
  end
end
puts qnum1
puts qnum2

if (qnum1 == 0 ) && (qnum2 == 0) 
    UI::messagebox ("ERROR - measured Tab Height not expected value.")
    Sketchup.undo    
end
if (qnum1 != 0 ) && (qnum1 == qnum2)
    UI::messagebox ("ERROR - I can't decide which part is which?")
    Sketchup.undo
end
if (qnum2 > qnum1) 
gt = g2
gs = g1
end

if (qnum2 < qnum1)  
gt = g1
gs = g2 
end

gt_ent = gt.entities
bc = gt.bounds.center
diff = 0
merror = 1

gt_ent.each do |e|
    if e.is_a? Sketchup::Edge
    point1 = e.start.position.transform!(gt.transformation)
    point2 = e.end.position.transform!(gt.transformation)
        if ((e.length == $kyyu_s4t) && (bb.contains? point1))
			normal = point1.vector_to point2
			plane1 = [point1,normal]
			plane2 = [point2,normal]
			d1 = bc.distance_to_plane plane1
			d2 = bc.distance_to_plane plane2
			diff = d2 - d1
				if diff < 0
					@ky_top_plane = plane1
					bottom_plane = plane2
				end	
				if diff > 0
					@ky_top_plane = plane2
					bottom_plane = plane1
				end     			
			puts diff      
        end
    end
# if (diff == $kyyu_s4t) then break end
# if (diff == -$kyyu_s4t) then break end
zdiff = $kyyu_s4t - diff.abs
if zdiff.abs < 0.001 then break end

end



tabnum = 0
gs_ent = gs.entities


gt_ent.each do |e|
    if e.is_a? Sketchup::Face
		ver = e.vertices
		vp1 = ver[0].position.transform!(gt.transformation)
		vp2 = ver[1].position.transform!(gt.transformation)
		vp3 = ver[2].position.transform!(gt.transformation)
		vp4 = ver[3].position.transform!(gt.transformation)
		if ((vp1.on_plane? @ky_top_plane) && (vp2.on_plane? @ky_top_plane) && (vp3.on_plane? @ky_top_plane) && (vp4.on_plane? @ky_top_plane))
			tabnum = tabnum + 1
			sface = ent.add_face(vp1, vp2, vp3, vp4)
            tabnormal = sface.normal
			sface_edges = sface.edges
			int = gs_ent.intersect_with(true, gs.transformation, gs_ent, gs.transformation, false, sface_edges)
            
			intfaces= []
			int.each {|e| intfaces = intfaces + e.faces}
			testfaces = intfaces.uniq       # All the unique possible faces
                #puts testfaces.length
            
            largest_area = 0
            largest_face = []
            count = testfaces.length
            for x in 0...count
                    face_area = testfaces[x].area
                if  face_area > largest_area
                       largest_area = face_area
                       largest_face[0] = testfaces[x]   
                end
            end
            picked = testfaces - largest_face
                #puts picked.length
            picked.each {|e| e.pushpull -$kyyu_s4t}   
            sface_edges.each {|e| e.erase!}
            # invalidated_view = view.invalidate
            # UI::messagebox ("check")
        end
    end
  
end
   #UI::messagebox ("bb check")
   Sketchup.active_model.active_entities.transform_entities(bbtr,selg[0],selg[1])
    sel.clear
Sketchup.active_model.commit_operation

    # Uncomment the following 3 lines of code, if you want the tab part move out of the way so
            # you can check the finished slots.  When done checking, undo will place the part back into position.
#dist = Geom::Point3d.new [0, 0, -0.125]
#movetr = Geom::Transformation.new (dist)
#Sketchup.active_model.active_entities.transform_entities(movetr,movegroup)

# Comment this out, if you don't want the Move Tool automatically selected when the plugin finishes.
Sketchup.send_action "selectMoveTool:"
 
puts "There are #{tabnum} tabs.\n "
end  #kyyu_Slots_4_Tabs

if( not file_loaded?('ky_Slots_4_Tabs.rb') )
	if $kyyu_context_submenu == 1
		UI.add_context_menu_handler do |menu|
			$kyyu_ConName.add_item('Slots 4 Tabs') { kyyu_Slots_4_Tabs }
		end
	else
		UI.add_context_menu_handler do |menu|
			$kyyu_ConName=menu.add_submenu('[kyyu]')
			$kyyu_ConName.add_item('Slots 4 Tabs') { kyyu_Slots_4_Tabs }
		end
		$kyyu_context_submenu = 1
	end
end   

#-----------------------------------------------------------------------------
file_loaded('ky_Slots_4_Tabs.rb')
